#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4290)
#endif

#include <OpenTracker/input/MyTutorialModule.h>
#include <OpenTracker/core/Context.h>
#include <iostream>


namespace ot {

//------------------------------------------------------------------------------
// In following macro, module registration function named
// "registerModuleMyTutorialModule()"  is defined.
// This would be called from SPLModule class.
//------------------------------------------------------------------------------
OT_MODULE_REGISTER_FUNC(MyTutorialModule){
  OT_MODULE_REGISTRATION_DEFAULT(MyTutorialModule, "MyTutorialConfig" );
}


//------------------------------------------------------------------------------
// FUNCTION: MyTutorialModlue()
//
// Constructor of this class
//------------------------------------------------------------------------------
MyTutorialModule::MyTutorialModule() 
{
  source = NULL;
  sink   = NULL;
  pushImageInterval = DEFAULT_PUSH_IMAGE_INTERVAL;

}


//------------------------------------------------------------------------------
// FUNCTION: MyTutorialModlue()
//
// Destructor of this class
//------------------------------------------------------------------------------
MyTutorialModule::~MyTutorialModule()
{

}


//------------------------------------------------------------------------------
// FUNCTION: createNode()
//
// This function creates node for "Sink" and "Source".
// Called when NaviTrack finds a name of node in XML file during starting up.
// Note that this implementation does not allow to have multiple nodes.
// Use std::vector to manage multiple nodes.
//------------------------------------------------------------------------------
ot::Node * MyTutorialModule::createNode( const std::string& name,  ot::StringTable& attributes)
{
  if( name.compare("MyTutorialSink") == 0 )
    {
      std::string strName=attributes.get("name");
      std::cout << "MyTutorialModule::createNode(): creating a MyTutorialSink node" << std::endl;
      std::cout << "MyTutorialModule::createNode(): attribute\"name\" is " << strName << std::endl;
      sink =  new MyTutorialSink();
      return sink;
    }
  if(name.compare("MyTutorialSource") == 0 )
    {
      std::string strName=attributes.get("name");
      std::cout << "MyTutorialModule::createNode(): creating a MyTutorialSource node" << std::endl;
      std::cout << "MyTutorialModule::createNode(): attribute\"name\" is " << strName << std::endl;
      source = new MyTutorialSource(strName);
      return source;
    }
  return NULL;
}


//------------------------------------------------------------------------------
// FUNCTION: pushState
//
// This function updates trackerinfo for all sources of this module
//------------------------------------------------------------------------------
void MyTutorialModule::pushEvent()
{
  static short* pix = NULL;

  if(source)
    {
//      std::cout << "MyTutorialModule::pushEvent() is called." << std::endl;
//      source->event.getPosition()[0] += 0.01;
//      source->event.getPosition()[1] += 0.02;
//      source->event.getPosition()[2] += 0.03;
//      source->event.getOrientation()[0] += 0.01;
//      source->event.getOrientation()[1] += 0.02;
//      source->event.getOrientation()[2] += 0.03;
//      source->event.getOrientation()[3] += 0.04;
//      
    }
}
  
  
//------------------------------------------------------------------------------
// FUNCTION: init()
//
// This function initializes the module.
//------------------------------------------------------------------------------
void MyTutorialModule::init(StringTable& attributes, ConfigNode * localTree)
{
  std::cout << "MyTutorialModule::init() is called." << std::endl;

  std::string strName=attributes.get("name");
  std::cout << "MyTutorialModule::init(): attribute \"name\" is " 
            << strName << std::endl;
}


//------------------------------------------------------------------------------
// FUNCTION: setTracker()
//
// This function updates trackerinfo for all sources of this module
//------------------------------------------------------------------------------
void MyTutorialModule::setTracker(std::vector<float> pos,std::vector<float> quat)
{
  std::cout << "MyTutorialModule::setTracker() is called." << std::endl;

  if (pos.size() != 3 || quat.size() != 4) {
    std::cout << "MyTutorialModule::setTracker(): illegal vector size." << std::endl;
    return;
  }

  if(source!=NULL)
    {
      ot::Event *event = new ot::Event();
      event->setAttribute("position",pos);
      event->setAttribute("orientation",quat);
      event->getButton()=0;
      event->getConfidence()=1.0;
      source->changed=1;
      event->timeStamp();
      source->updateObservers( *event );
    }
}


//------------------------------------------------------------------------------
// FUNCTION: setImage()
//
// This function updates image for all sources of this module
//------------------------------------------------------------------------------
void MyTutorialModule::setImage(Image& img, int w, int h)
{
  //static short* pix = NULL;
  
  if(source!=NULL)
    {
      // split a image
      int nlines  = MAXIMUM_IMAGE_SIZE / (w*sizeof(short));
      int nblocks = h / nlines;
      if (h % nlines > 0) nblocks ++;

      for (int n = 0; n < nblocks; n ++) 
        {
          int ysize;
          int endflag = 0;

          if (n == nblocks-1)
            {
              ysize = h-nlines*n;
              endflag = 1;
            }
          else
            ysize = nlines;

          //std::cerr << "endflag = " << endflag << std::endl;
          //std::cerr << "xsize   = " << w << std::endl;
          //std::cerr << "ysize   = " << ysize << std::endl;
          //std::cerr << "totalysize = " << h << std::endl;
          //std::cerr << "line = "     << n*nlines << std::endl;

          short* ba = (short*)img.image_ptr;
          Image fragment(w,ysize,sizeof(short),&ba[w*n*nlines]);
          

          //ot::Event event;
          ot::Event event;
          event.setAttribute("endflag",   endflag);
          event.setAttribute("xsize",     w);
          event.setAttribute("ysize",     ysize);
          event.setAttribute("totalysize",h);
          event.setAttribute("line",      n*nlines);
          event.setAttribute("image",     fragment);
          event.setAttribute("index",     n);

          event.timeStamp();
          source->updateObservers( event );
          this->context->loopOnce();
          //usleep(pushImageInterval);
          OSUtils::sleep(pushImageInterval);
        }
    }
}


} // end of "namespace ot"





