#include <OpenTracker/input/MyTutorialSink.h>

#ifdef WIN32
#include <windows.h>
#include <OpenTracker/misc/SPLOT_Win.h>
#endif

#include <iostream>

namespace ot {

  MyTutorialSink::MyTutorialSink()
  {
    position.resize(3);
    orientation.resize(4);

    position[0] = position[1] = position[2] = 0.0;
    orientation[0] = orientation[1] = orientation[2] = orientation[3] = 0.0;

    width = height = 0;
    width_buf = height_buf = 0;
    image = NULL;
    image_buf = NULL;

    timestamp = 0.0;
    
  }

  MyTutorialSink::~MyTutorialSink()
  {
  }

  void MyTutorialSink::onEventGenerated( Event& event, Node& generator)
  {
    //std::cerr << "MyTutorialSink::onEventGenerated()" << std::endl;
    if (event.hasAttribute("position"))
      for(int i = 0; i < 3; i ++)
        position[i]=event.getPosition()[i];
    else
      {
	position[0]=0.0;
	position[1]=0.0;
	position[2]=0.0;
      }

    if (event.hasAttribute("orientation"))
      {
        for  (int i = 0; i < 4; i ++) {
          orientation[i]= event.getOrientation()[i];
        }
      }
    else
      {
	orientation[0]=0.0;
	orientation[1]=0.0;
	orientation[2]=0.0;
	orientation[3]=0.0;
      }
    

    if (event.hasAttribute("image") &&
        event.hasAttribute("xsize") &&
        event.hasAttribute("ysize") &&
        event.hasAttribute("totalysize") &&
        event.hasAttribute("line") //&&
        //event.hasAttribute("endflag")
        )
      {
        Image segimg   = event.getAttribute((Image*)NULL, "image");
        width_buf      = event.getAttribute(std::string("xsize"),0);
        height_buf     = event.getAttribute(std::string("totalysize"),0);
        int segheight  = event.getAttribute(std::string("ysize"),0);
        int line       = event.getAttribute(std::string("line"),0);
        int endflag    = event.getAttribute(std::string("endflag"), 0);
        int index      = event.getAttribute(std::string("index"), 0);

        //std::cerr << "width_buf   = " << width_buf  << std::endl;
        //std::cerr << "height_buf  = " << height_buf << std::endl;
        //std::cerr << "segheight   = " << segheight  << std::endl;
        //std::cerr << "line        = " << line       << std::endl;
        //std::cerr << "endflag     = " << endflag    << std::endl;
        std::cerr << "Fragment index = " << index      << std::endl;
        if (endflag) {
          std::cerr << "end of image." << std::endl;
        }

        if (!image_buf) {
          image_buf = new Image();
          image_buf->SetSize(width_buf, height_buf, sizeof(short));
          image_buf->image_ptr = (void*)malloc(image_buf->size());
        } else if (image_buf->size() != width_buf * height_buf * sizeof(short)) {
          free(image_buf->image_ptr);
          image_buf->SetSize(width_buf, height_buf, sizeof(short));
          image_buf->image_ptr = (void*)malloc(image_buf->size());
        }
        short* dest = (short*)image_buf->image_ptr;

        if (width_buf*segheight > 0)
          memcpy(&dest[width_buf*line], segimg.image_ptr, width_buf*segheight*sizeof(short));
        if (endflag)
          {
            // switch image buffer
            Image* tmp;
            int    tmp_width, tmp_height;
            tmp       = image;
            image     = image_buf;
            image_buf = tmp;

            tmp_width = width;
            width     = width_buf;
            width_buf = tmp_width;

            tmp_height= height;
            height    = height_buf;
            height_buf= tmp_height;

            timestamp = event.time;
          }
        
      }
    else
      {
        width  = 0;
        height = 0;
      }


  }


} // end of namespace ot
