#undef OPENTRACKER_EXPORTS
#include <OpenTracker/OpenTracker.h>
#include <OpenTracker/input/SPLModules.h>
#include <OpenTracker/common/CallbackModule.h>
#include <OpenTracker/input/MyTutorialModule.h>
#include <iostream>
#include <fstream>
#include <exception>
#include <stdlib.h>
#include <string.h>

#define MAX_NUM_IMAGE 20

// just for testing
#include <sys/time.h>


using namespace std;
using namespace ot;

#define DEFAULT_LOOP_INTERVAL    100  // ms


// Callback function for NaviTrack event
void callbackFunc(Node&, Event &, void *);


int main(int argc, char **argv)
{
  char* xmlfile;
  long  interval;

  // set parameters based on arguments
  if (argc <= 1) {
    cerr << "Usage: " << argv[0] << " <OT XML> [<interval in ms>]"  << endl;
    exit(-1);
  }
  xmlfile = argv[1];
  if (argc > 2) {
    interval = atoi(argv[2]);
  } else {
    interval = DEFAULT_LOOP_INTERVAL;
  }
  cerr << "XML File     : " << xmlfile << endl;
  cerr << "Loop interval: " << interval << "ms" << endl;


  // context loop
  addSPLModules();

  Context context(1);
  context.parseConfiguration(xmlfile);
  context.start();

  CallbackModule* cbm = (CallbackModule*) context.getModule("CallbackConfig");
  if (cbm == NULL) {
    cerr << "Failed to load CallbackConfig Module." << endl;
  }
  
  cbm->setCallback( "cb1", (OTCallbackFunction*)&callbackFunc , NULL);
  context.runAtRate(1000.0/(double)interval);

  context.close();

}


void callbackFunc(Node&, Event &event, void *)
{
  struct timeval tm1, tm2;

  vector<float> position(3, 0.0);
  vector<float> orientation(4, 0.0);


  gettimeofday(&tm1, NULL);
  
  cerr << "Event generated" << endl;
  if (event.hasAttribute("position")) {
    for(int i = 0; i < 3; i ++)
      position[i]=event.getPosition()[i];
  } else {
    position[0]=0.0;
    position[1]=0.0;
    position[2]=0.0;
  }
  
  if (event.hasAttribute("orientation")) {
    for  (int i = 0; i < 4; i ++) {
      orientation[i]= event.getOrientation()[i];
    }
  } else {
    orientation[0]=0.0;
    orientation[1]=0.0;
    orientation[2]=0.0;
    orientation[3]=0.0;
  }

  cout << "Received position = ("
            << position[0] << ", " << position[1] << ", " << position[2] << "),  ";
  cout << "orientation = ("
            << orientation[0] << ", " << orientation[1] << ", " << orientation[2] << ", "
            << orientation[3] << ")"  << endl;
  
  gettimeofday(&tm2, NULL);
  long usec;
  usec = (tm2.tv_sec - tm1.tv_sec) * 1000000;
  usec = (usec + tm2.tv_usec) - tm1.tv_usec;
  cerr << "Callback completed in " << usec << "(us)." << endl;
  
}


