#undef OPENTRACKER_EXPORTS
#include <OpenTracker/OpenTracker.h>
#include <OpenTracker/input/SPLModules.h>
#include <OpenTracker/input/MyTutorialModule.h>
#include <iostream>
#include <fstream>
#include <exception>
#include <stdlib.h>
#include <string.h>

#include <OpenTracker/types/Image.h>

#include "pgmimage.h"


using namespace std;
using namespace ot;


#define DEFAULT_LOOP_INTERVAL    3000 // ms
#define NUM_IMAGE                5

int main(int argc, char **argv)
{
  char* xmlfile;
  long  interval;

  // set parameters based on arguments
  if (argc <= 1) {
    cerr << "Usage: " << argv[0] << " <OT XML> [<interval in ms>]"  << endl;
    exit(-1);
  }
  xmlfile = argv[1];
  if (argc > 2) {
    interval = atoi(argv[2]);
  } else {
    interval = DEFAULT_LOOP_INTERVAL;
  }
  cerr << "XML File     : " << xmlfile << endl;
  cerr << "Loop interval: " << (interval) << "ms" << endl;

  // read images
  PgmImage pi[10];

  for (int i = 0; i < NUM_IMAGE; i ++) {
    char filename[50];
    sprintf(filename, "human-%02d.pgm", i);

    std::ifstream fin(filename, std::ios::binary);
    fin >> pi[i];
    fin.close();
  }

  // context loop
  addSPLModules();

  Context context(1);
  context.parseConfiguration(xmlfile);
  context.start();

  MyTutorialModule* mtm = (MyTutorialModule*) context.getModule("MyTutorialConfig");
  mtm->setPushImageInterval(100);

  int stopflag = 0;
  int i = 0;
  while (stopflag == 0) {

    int w, h;
    pi[i].getSize(&w, &h);

    Image img(w, h, sizeof(short), (char*)pi[i].getDataS());

    if (mtm) {
      mtm->setImage(img, w, h);
    }

    stopflag = context.loopOnce();

    OSUtils::sleep(interval);
    i ++;
    i = i % NUM_IMAGE;
  }

  context.close();

}


