/*============================================================*
 *                                                            *
 * Simple Image Canvas on GTK+                                *
 *                                                            *
 * Copyright (C) 2007, Junichi Tokuda.                        *
 *                                                            *
 *============================================================*
 * Description: Simple Image Display class		      *
 * Created on:  10 May, 2007				      *
 *============================================================*/


/*============================================================
 * Example to use the class:
 *============================================================
 *
 * #include <iostream>
 * #include "simplecanvas.h"
 *
 * int main(int argc, char* argv[])
 * {
 *   SimpleCanvas sc(256, 256);
 *   unsigned char a[256*256];
 * 
 *   // generate bitmap
 *   for (int i = 0; i < 256; i ++) {
 *     for (int j = 0; j < 256; j ++) {
 *       a[j*256+i] = j/2;
 *     }
 *   }
 * 
 *   sc.run();     // start simple canvas thread
 *
 *   sleep(2);
 *
 *   sc.load(a);   // load a new bitmap
 *   sc.update();  // update screen
 *
 *   sleep(2);
 *
 *   sc.stop();    // stop the thread
 * }
 *
 *============================================================*/

 
#ifndef _INC_SIMPLE_CANVAS
#define _INC_SIMPLE_CANVAS

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <math.h>
#include <pthread.h>

class SimpleCanvas {

private:
  pthread_t  thread;

  guchar *drawbuf;

  GtkWidget  *drawing_area;
  GtkWidget  *window;
  GtkWidget  *vbox, *hbox;
  GtkWidget  *button;
  GSList *group;
  int  width, height, depth, size;

private:
  static SimpleCanvas* pointer;
  static gint onDraw(GtkWidget *widget,GdkEventExpose *event);

public:
  SimpleCanvas(int, int);
  ~SimpleCanvas();
  
  void load(short*);
  void load(unsigned char*);
  void update();

  static void*  process(void*);
  void          run();
  void          stop();

};

#endif //_INC_SIMPLE_CANVAS
